
# THIS FILE IS GENERATED FROM wofrysrw SETUP.PY
short_version = '1.0.16'
version = '1.0.16'
full_version = '1.0.16.dev0+f46b337'
git_revision = 'f46b33709ee9592284919582f610d37af7448970'
release = False

if not release:
    version = full_version
    short_version += ".dev"
